# =====================================================================
# csa-sendmail.awk: shell front-end to the _sendmail() AWK function.
#
# Copyright (c) 2004,2006 Carlo Strozzi
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================

BEGIN {

  # Check context and arguments, and set defaults.

  if (tpl !~ /^\//) exit(rc=1)				# no default!

  _TBLS[0,0] = split(tbl,tb,",")

  for (i=1; i<=_TBLS[0,0]; i++) {

      if (tb[i] !~ /^\//) tb[i] = "/dev/null"		# full-path ?

      # Load extra NoSQL tables if requested. Table rows will be assigned
      # to _TBLS[1], _TBLS[2], etc., with _TBLS[0,0] being the no. of tables
      # to load and _TBLS[n,0] is the no. of records in table 'n'. Any table
      # header information is stored in the separate uni-dimensional array
      # _TBLSH[]. For backward compatibility, the first (or only) of the
      # specified tables is loaded also in the _TBL[] uni-dimensional array,
      # and the relevant table header is stored in the _TBLH scalar.
      # Note that splitting table records into fields and unescaping NoSQL
      # special sequences is up to whoever will make use of these values,
      # often through the application-level _userproc() function.

      if (tb[i] != "") {
         _TBLS[i,0] = 0
	 if (i==1) _TBL[0] = 0				# compatibility
         while (getline record < tb[i] > 0) {
	    # Store table header in a separate place.
	    if (record ~ /^\001/ && _TBLSH[i] == _NULL) {
	       _TBLSH[i] = record
	       if (i==1) _TBLH = record			# compatibility
	       continue
	    }
	    # Append raw table record to _TBLS[n].
	    _TBLS[i,++_TBLS[i,0]] = record
	    if (i==1) _TBL[++_TBL[0]] = record		# compatibility
         }

         close(tb[i])
      }
  }

  for (j in ENVIRON) {

      # Only CSA_xxx variables are automatically encoded, while all
      # others are left up to the application-level processor.
      # Note that this is left solely for backwad compatibility, as
      # with the new encoding flags it is no longer necessary.

      if (j ~ /^CSA_/) _response(j,ENVIRON[j])
  }

  # Call the application-level processor. To prevent _userproc()
  # from being called twice, with unpredictable results, the call to
  # _userproc() is skipped if we are alredy being called as part of
  # write-back statements written by _userproc() to CSA_TPLWB during
  # the first-level template response processing.

  if (ENVIRON["CSA_TPLWB"] == _NULL) _userproc(_O_RESPONSE)

  exit(rc=_sendmail(tpl,eval,trim,0,norepl))
}

# EOF
