# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# string _gregorian(float jdate [, char fmt, boolean short])
#
# Convert Julian date to calendar date. Algorithm adopted from Press
# et al.
#
# The "fmt" argument can be one of:
#
# c : "computer" format : yyyymmddHHMMSS
# i : "iso" format      : yyyy-mm-dd HH:MM:SS
# e : "europe" format   : dd/mm/yyyy HH:MM:SS
# u : "us" format       : mm/dd/yyyy HH:MM:SS
#
# The "short" argument tells the function that the year is expressed in
# two-digit format.
#
# =====================================================================

function _gregorian(jd,fmt,short,		j1,j2,j3,j4,j5,intgr,\
						gregjd,tmp,dayfrac,\
						frac,hr,mn,sc,f,y,m,d) {
   if (fmt == "") fmt = "c"

   y_fmt = "%04d"

   if (short) y_fmt = "%02d"
   else y_fmt = "%04d"

   # Make sure date casts to number, or rounding errors may occur.
   jd += 0

   if (!jd) return _EINVAL			# zero, or not a number.

   # get the date from the Julian day number

   intgr   = int(jd)
   frac    = jd - intgr
   gregjd  = 2299161

   if (intgr >= gregjd) {               # Gregorian calendar correction
      tmp = int(((intgr - 1867216) - 0.25) / 36524.25)
      j1 = intgr + 1 + tmp - int(0.25*tmp)
   }  else j1 = intgr

   # correction for half day offset
   dayfrac = frac + 0.5
   if (dayfrac >= 1.0) {
      dayfrac -= 1.0
      ++j1
   }

   j2 = j1 + 1524
   j3 = int(6680.0 + ((j2 - 2439870) - 122.1)/365.25)
   j4 = int(j3*365.25)
   j5 = int((j2 - j4)/30.6001)

   d = int(j2 - j4 - int(j5*30.6001))
   m = int(j5 - 1)
   if (m > 12) m -= 12
   y = int(j3 - 4715)
   if (m > 2)   --y
   if (y <= 0)  --y

   #  get time of day from day fraction

   hr  = int(dayfrac * 24.0)
   mn  = int((dayfrac*24.0 - hr)*60.0)
         f = ((dayfrac*24.0 - hr)*60.0 - mn)*60.0
   sc  = int(f)
   f -= sc

   if (f > 0.5) {
      ++sc
      if (sc == 60) {
	 mn++
	 sc = 0
      }
   }

   if (y < 0) y = -y

   if (fmt == "c") {
      fmt = y_fmt "%02d%02d%02d%02d%02d"
      return sprintf(fmt,y,m,d,hr,mn,sc)
   }
   else if (fmt == "i") {
      fmt = y_fmt "-%02d-%02d %02d:%02d:%02d" 
      return sprintf(fmt,y,m,d,hr,mn,sc)
   }
   else if (fmt == "e") {
      fmt = "%02d/%02d/" y_fmt " %02d:%02d:%02d"
      return sprintf(fmt,d,m,y,hr,mn,sc)
   }
   else {
      # US-style date.
      fmt = "%02d/%02d/" y_fmt " %02d:%02d:%02d"
      return sprintf(fmt,m,d,y,hr,mn,sc)
   }
}
