# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2007 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================

function _uncgi(s,o_file,auth,rdfmap,		       a,b,i,n,o_tmp) {

   # rdfmap is "0" for "scalar", "1" for RwA resource, "2" for URL.

   if (s == "") return			# Nothing to do.

   if (o_file == "") o_file = _STDOUT	# set default

   # If input data contains assignments it is truncated to 10 KB to
   # prevent buffer and environment overflows in the calling program.
   # Actually, the input is coming from both stdin and QUERY_STRING, so
   # this check should be improved to account for both, to become really
   # effective.
   # On the contrary, if input does not contain assignments I consider
   # it an ISINDEX query and assign its value to the CSA_ISINDEX
   # variable. In this case I truncate the value to 512 bytes, although
   # it should already have been truncated to something shorter by the
   # Web server itself.

   if (s ~ /=/) {
      if (ENVIRON["CONTENT_LENGTH"] > _MAXSIZE)
	 return _EMSGSIZE		# Max. input data size exceeded.
   }
   else {
      s = "CSA_ISINDEX=" substr(s,1,512)
      gsub(/&/, "%26", s)
      gsub(/\?/, "%3F", s)
   }

   # remove trailing assignments that have been invalidated by
   # the truncation process
   #sub(/&[^&=]*$/, "", s)
   sub(/[?&][^?&=]*$/, "", s)
   sub(/%.?$/, "", s)

   # single-quotes must be doubled for rc(1),
   # whether they are URI-escaped or not.
   ###gsub(/'/, "''", s)
   ###gsub(/%27/, "''", s)

   # split-up the input assignments
   #if ((n = split(s, a, "\\&")) == 0) return
   if ((n = split(s, a, /[?&]/)) == 0) return

   # Note: a construct like "for (i in a)" may not preserve ordering.
   while (++i <= n) {

       # split each "name=value" pair
       split(a[i],b,"=")

       # Handle virtual headers.
       if (b[1] ~ /^[Xx]-/) {
	  gsub(/-/, "_", b[1])
	  b[1] = toupper(b[1])
       }

       # normalize white-space and line-breaks a-la uncgi,
       # and remove leading "." from names. Strip characters
       # that would make textarea names look like virtual headers.
       # Unlike uncgi, for a variable to be normalized it must
       # begin with one dot.

       if (sub(/^\.([Xx]-)?/, "", b[1])) {
	  sub(/^[ \t]*/,"",b[1])
	  sub(/[ \t]*$/,"",b[1])
	  gsub(/%0D%0A/, "%0A", b[2])
	  gsub(/%0D/, "%0A", b[2])
       }

       # replace invalid characters with underscores (_) in variable names

       gsub(/[^A-Za-z0-9_]/, "_", b[1])

       # skip variable names that begin with a number followed by  
       # an underscore, as they are reserved for CSA internal use
       # (for instance to generate the file-name associated with
       # a file-type argument).

       if (b[1] ~ /^[0-9]+_/) continue

       b[2] = _uridecode(b[2])

       # extract HTTP authorization if requested. To cope with how
       # HTTP cookies are handled by browsers, in case of multiple
       # assignments only the first one is retained here.

       if (toupper(b[1]) == "AUTHORIZATION") {
	  if (auth) _authset(b[2],1,o_file)
	  auth = 0		 # ignore any further re-assignments.
       }

       # Input values exceeding '_MAXARG' are saved to files rather than
       # in storage, to prevent bloating the application program's shell
       # environment. It will be up to CSA application programs to look
       # for data in temporary files when they do not find it in the
       # expected scalar values. Note that in theory also the size of the
       # 'b[1]' string may be too big, so we must account for that too.

       else if (length(b[1]) > _MAXARG) return _EMSGSIZE

       # Exclude RwA as a whole from size checks, as it is not going
       # to end up in the underlying shell environment and sending it
       # to a file would be more trouble than it is worth. Individual
       # assignments inside it will be checked for size by _rwa2csa()
       # later on. Note that unlike the XML-RPC/SOAP and the RwA cases
       # (see csaCgi.awk and _rwa2csa() respectively) we can make no
       # assumptions here regarding whether the input data is XML-encoded
       # or not, so we must store it as-is. It will be up to the
       # application program to know what to expect and behave
       # appropriately.

       else if (b[1] == "CSA_RWA") _cgivar(b[1], b[2], rdfmap)

       else if (length(b[2]) > _MAXARG) {

	 # The file name must be built according to CSA-RPC specs.
	 o_tmp = _TMPSTEM ".file."  b[1] ".arg." b[1] ".tmp"
	 _UPLOAD[o_tmp] = o_tmp
	 _cgivar(b[1],o_tmp,1)
	 printf("%s",b[2]) > o_tmp
	 close(o_tmp)
	 _TOOBIG[o_tmp] = b[2]
       }

       else if ((_ARGSIZE += (length(b[1]) + length(b[2]))) > _MAXARG)
							return _EMSGSIZE

       else _cgivar(b[1], b[2], rdfmap)		# do the assignment
   }
}

