# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2008 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# string _uriencode(string s, boolean mode)
#
# Perform URI-encoding of special characters, according to RFC 1378.
# See http://www.apache.org/info/css-security/ for more info on
# the so-called Cross-Site Scripting vulnerabilities. If the 'mode'
# argument is set to '1', then the encoding is done twice for some of
# the involved characters. This is meant to produce values that are to
# be included in PATH_INFO elements, rather than in the QUERY_STRING,
# as the content of PATH_INFO is unencoded a first time by the CGI
# interface itself (verified with both Apache and Boa), and then by the
# CSA _uncgi() function. If data contains characters such as / ? and
# &, and they are encoded only once, they will be made available to CSA
# already unencoded, and that would disrupt a number of things (for
# instance, splitting PATH_INFO on slashes would become unreliable, to
# name only one).
#
# It is VERY important to have a look at RFC-3986 to understand how
# percent-encoding should be done. On page 13 of said RFC-3986
# (http://asg.web.cmu.edu/rfc/rfc3986.html) are listed a few reserved
# characters that shoud never be encoded in URIs, i.e.:
#
#   reserved    = gen-delims / sub-delims
#
#   gen-delims  = ":" / "/" / "?" / "#" / "[" / "]" / "@"
#
#   sub-delims  = "!" / "$" / "&" / "'" / "(" / ")"
#		/ "*" / "+" / "," / ";" / "="
#
# This function should be inspected for compliance with RFC-3986 since
# there will probably be things that need to be changed/improved. Also,
# at the moment the encoding process is based on tabular data, while
# it may be preferable to use the relevant SWU AWK library functions
# for this purpose in the future.
# =====================================================================

function _uriencode(string,mode,		a,i,j) {

   if (mode == _O_PATHINFO) {
      gsub(/&/, "%26", string)
      gsub(/\?/, "%3F", string)
      gsub(/=/, "%3D", string)
      gsub(/\//, "%2F", string)
   }

   i = split(string,a,_NULL); string = _NULL

   for (j=1; j<=i; j++) {

       # allow rc(1) lists to leak-through.
       if (a[j] ~ /[a-zA-Z0-9]/ || a[j] == "\001") string = string a[j]

       # unmapped characters are ignored.
       else if (_MAPCHAR[a[j]] == _NULL) continue

       else string = string "%" _MAPCHAR[a[j]]
   }

   return string
}

